/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.iit.genomics.cru.isomirage;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.swing.SwingWorker;

/**
 * @(#)CountIsoforms.java
 * 11 Sept 2014
 * Copyright 2014 Computational Research Unit of IIT@SEMM. All rights reserved.
 * Use is subject to MIT license terms.
 * 
 * Swing worker class for counting isoform occurrences.
 * 
 * @author Heiko Muller
 * @version 1.0
 * @since 1.0
 */
public class CountIsoforms extends SwingWorker{
    
    /**
     * Path to genome file
     */
    String isoformfile;
    
    /**
     * Path to Bowtie map file for a given sample
     */
    String bowtiefile;
    
    /**
     * Hashtable for Isoform objects
     */
    Hashtable<String, Isoform> hash;

    /**
     * Processes Bowtie map file in the background
     * @return Object
     * @throws Exception 
     */
    @Override
    protected Object doInBackground() throws Exception {
        //throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        String result = "";
        initHash();
        try{
            processBowtieMap();
            dumpHashToFile(new File(bowtiefile));
        }catch(IncompatibleGenomeException ige){
            result = ige.getKey() + " not found in genome.";
            
        }
        return result;
    }
    
    
    /**
     * Constructor for CountIsoforms
     * @param isoformfile
     * @param bowtiefile 
     */
    public CountIsoforms(String isoformfile, String bowtiefile) {
        this.isoformfile = isoformfile;
        this.bowtiefile = bowtiefile;
    }
   
    /**
     * Inits hash by adding a new Isoform object for each distinct sequence in the Bowtie map file
     */
    private void initHash(){
        hash = new Hashtable<String, Isoform>();
        try{
            FileReader fr = new FileReader(isoformfile);
            BufferedReader br = new BufferedReader(fr);
            String line = "";
            while((line = br.readLine()) != null){
                String[] temp = line.split("\t");
                if(!hash.containsKey(temp[1])){
                    hash.put(temp[1], new Isoform(temp[0], temp[1], 0));
                }
                
            }
            
            fr.close();
            br.close();
        }catch(IOException ioe){
            ioe.printStackTrace();
        }
    }
    
    /**
     * Reads Bowtie map file and carries out counting of Isoform occurrences
     * @throws IncompatibleGenomeException 
     */
    private void processBowtieMap() throws IncompatibleGenomeException{
        
        try{
            FileReader fr = new FileReader(bowtiefile);
            BufferedReader br = new BufferedReader(fr);
            String line = "";
            while((line = br.readLine()) != null){
                String[] temp = line.split("\t");
                //if(temp.length > 3 && temp[3].equals("0")){
                if(temp.length > 3){
                    if(hash.containsKey(temp[4])){
                        hash.get(temp[4]).add();
                    }else{
                        //System.out.println(temp[0] + "\t" + temp[4] + " not found");
                        //throw new IncompatibleGenomeException(temp[0] + "\t" + temp[4]);
                    }
                }
            }
            fr.close();
            br.close();
        }catch(IOException ioe){
            ioe.printStackTrace();
        }
    }
    
    /**
     * Dumps hash content to console
     */
    private void dumpHash(){
        Object[] keys = hash.keySet().toArray();
        for(int i = 0; i < keys.length; i++){
            Isoform iso = hash.get(keys[i]);
            System.out.println(iso.name + "\t" + iso.count);
        }
    }
    
    /**
     * Dumps hash content to file
     * @param inputfile 
     */
    private void dumpHashToFile(File inputfile){
        File out = new File(inputfile.getParentFile() + File.separator + inputfile.getName() + "_isoforms.txt");
        try{
        FileWriter fwr = new FileWriter(out, false);
        fwr.write(inputfile + "\r\n");
            Object[] keys = hash.keySet().toArray();
            for(int i = 0; i < keys.length; i++){
                Isoform iso = hash.get(keys[i]);
                fwr.write(iso.name + "\t" + iso.count + "\r\n");
            }
            fwr.write("\r\n");
            fwr.flush();
            fwr.close();
        }catch(IOException ioe){
            ioe.printStackTrace();
        }
    }
    
    /**
     * Combines isoform files for each sample into a common table
     * @param mapfiles 
     */
    public static void parse(ArrayList<String> mapfiles){
        ArrayList<ArrayList<String>> al = new ArrayList<ArrayList<String>>();
        try{
            for(int i = 0; i < mapfiles.size(); i++){
                FileReader fr = new FileReader(mapfiles.get(i) + "_isoforms.txt");
                BufferedReader br = new BufferedReader(fr);
                String line = "";
                ArrayList<String> alt = new ArrayList<String>();
                while((line = br.readLine()) != null){
                    alt.add(line);
                }
                fr.close();
                br.close();
                al.add(alt);
                
            }
            FileWriter fwr = new FileWriter(mapfiles.get(0) + "_linked.txt");
            int limit = al.get(0).size();
            for(int i = 0; i < limit; i++){    
                
                    for(int j = 0; j < al.size(); j++){
                        if(j == 0){
                            if(i == 0){
                                File f = new File(al.get(j).get(i));
                                String sample = f.getName();
                                sample = sample.replace(".map", "");
                                fwr.write("\t" + sample);
                            }else{
                                fwr.write(al.get(j).get(i) + "\t");
                            }
                        }else{
                            if(i == 0){
                                File f = new File(al.get(j).get(i));
                                String sample = f.getName();
                                sample = sample.replace(".map", "");
                                fwr.write("\t" + sample);
                            }else{
                                String s = al.get(j).get(i);
                                String[] sa = s.split("\t");
                                if(sa.length > 1){
                                    fwr.write(sa[1] + "\t");
                                }
                            }
                        }
                    }
                
                fwr.write("\r\n");
            }
            fwr.flush();
            fwr.close();
        }catch(IOException ioe){
            ioe.printStackTrace();
        }
    }
}
