/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.iit.genomics.cru.isomirage;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

/**
 * @(#)RPKM.java
 * 11 Sept 2014
 * Copyright 2014 Computational Research Unit of IIT@SEMM. All rights reserved.
 * Use is subject to MIT license terms.
 * 
 * Performs RPKM normalization of occurrence counts.
 * 
 * @author Heiko Muller
 * @version 1.0
 * @since 1.0
 */
public class RPKM {
    
    /**
     * row ids
     */
    String[] ids = null;
    
    /**
     * counts data
     */
    double[][] data = null;
    
    /**
     * normalized counts data
     */
    double[][] normalized_data = null;
    
    /**
     * column sums of counts
     */
    double[] sums = null;
    
    /**
     * header
     */
    String header = "";
    
    /**
     * Constructor of RPKM class
     * 
     * @param filename input counts file
     */
    public RPKM(String filename){
        readFile(filename);
        generateNormalizedData();
        writeOutput(filename);
    }
    
    /**
     * Reads input data, inits and fills data table
     * @param filename 
     */
    private void readFile(String filename){
        int rows = readFileLineNumber(filename) - 1;//minus header line
        int current = 0;
        ids = new String[rows];
        try{
            FileReader fr = new FileReader(filename);
            BufferedReader br = new BufferedReader(fr);
            header = br.readLine();
            String line = "";
            line = br.readLine();
            String[] temp = line.split("\t");
            data = new double[rows][(temp.length - 1)];
            normalized_data = new double[rows][(temp.length - 1)];
            sums = new double[temp.length - 1];
            ids[current] = temp[0];
            for(int i = 1; i < temp.length; i++){                
                data[current][(i - 1)] = Double.parseDouble(temp[i]);

            }
            while((line = br.readLine()) != null && line.trim().length() > 0){
                current++;
                //System.out.println(current);
                temp = line.split("\t");
                ids[current] = temp[0];
                for(int i = 1; i < temp.length; i++){
                    data[current][(i - 1)] = Double.parseDouble(temp[i]);
                }
            }

        fr.close();
        br.close();
        }catch(IOException ioe){
        }
    }
    
    /**
     * Reads the number of lines in input file
     * @param filename
     * @return int line number
     */
    private int readFileLineNumber(String filename){
         int result = 0;
        try{
            FileReader fr = new FileReader(filename);
            BufferedReader br = new BufferedReader(fr);
            String line = "";
            while((line = br.readLine()) != null && line.trim().length() > 0){
                result++;
            }
        }catch(IOException ioe){}
        return result;
     }
    
    /**
      * Generates normalized data
      */
    private void generateNormalizedData(){
        getColumnsSums();
        for(int i = 0; i < data.length; i++){ 
            for(int j = 0; j < data[0].length; j++){
                normalized_data[i][j] = data[i][j]/sums[j] * 1000000;
            }
        }
    }
    
    /**
     * Calculates column sums
     */
    private void getColumnsSums(){
        for(int i = 0; i < data.length; i++){
            for(int j = 0; j < data[0].length; j++){
                sums[j] = sums[j] + data[i][j];
            }
        }
    }
    
    /**
      * Writes normalized data output to file.
      * Output file is located in the directory of the input file and has an _RPKM.txt extension.
      * @param filename 
      */
    private void writeOutput(String filename){
        try{
            FileWriter fwr = new FileWriter(filename + "_RPKM.txt");
            fwr.write(header + "\r\n");
            for(int j = 0; j < normalized_data.length; j++){
                if(sum(data[j]) > 0){
                    fwr.write(ids[j] + "\t");
                    for(int i = 0; i < normalized_data[0].length; i++){
                        fwr.write(normalized_data[j][i] + "\t");
                    }
                    fwr.write("\r\n");
                }
            }
            fwr.flush();
            fwr.close();
        }catch(IOException ioe){

        }
     }
    
    private double sum(double[] d){
        double result = 0;
        for(int i = 0; i < d.length; i++){
            result = result + d[i];
        }
        return result;
    }
    
}
